<?php

namespace Drupal\simple_analytics;

/**
 * Simple Analytics Helper functions.
 */
class SimpleAnalyticsHelper {

  /**
   * Get Configuration Name.
   */
  public static function getConfigName() {
    return 'simple_analytics.settings';
  }

  /**
   * Get Configuration Object.
   *
   * @return \Drupal\Core\Config\Config|\Drupal\Core\Config\ImmutableConfig
   *   The configuration object.
   */
  public static function getConfig($editable = FALSE) {
    if ($editable) {
      $config = \Drupal::configFactory()->getEditable(static::getConfigName());
    }
    else {
      $config = \Drupal::config(static::getConfigName());
    }
    return $config;
  }

  /**
   * Check the tracking conditions.
   *
   * @return bool
   *   TRUE = Excluded.
   */
  public static function checkNotrackConditions($config = NULL) {

    if (!$config) {
      $config = SimpleAnalyticsHelper::getConfig();
    }

    $track_admin = $config->get('track_admin');
    $track_auth = $config->get('track_auth');
    $track_exclude_url = $config->get('track_exclude_url');

    // No tracking for admin pages.
    $current_url = $_SERVER['REQUEST_URI'];
    if (!$track_admin && strstr($current_url, '/admin') !== FALSE) {
      return TRUE;
    }

    // No tracking for Authenticated Users.
    $current_user = \Drupal::currentUser();
    if (!$track_auth && $current_user->isAuthenticated()) {
      return TRUE;
    }

    // No tracking for excluded pages.
    if (!empty($track_exclude_url)) {
      foreach ($track_exclude_url as $url) {
        if (strpos($current_url, $url) !== FALSE) {
          return TRUE;
        }
      }
    }

    // No tracking for S.A system / API pages.
    $url = 'simple_analytics/api';
    if (strpos($current_url, $url) !== FALSE) {
      return TRUE;
    }

    return FALSE;
  }

}
